//
//  IndexPath+MINTLExtension.swift
//  TreeOutlinerSwift
//
//  Created by narita on 2017/10/14.
//  Copyright © 2017年 narita. All rights reserved.
//

import Foundation

extension IndexPath
{
    // 末尾のindexを削除して、削除したindexに+1した値を追加する。
    // ex: 選択範囲の次の行を示すのに使用する
    func indexPathByIncrementLastIndex() -> IndexPath
    {
        // IndexPathでNSIndexPathのindex(atPosition:)に相当するメソッドが見つからなかったので
        // NSIndexPathにキャストして使用している
        let theIncrementedLastIndex =
            (self as NSIndexPath).index(atPosition: self.count - 1) + 1

        let theResult = self.dropLast().appending(theIncrementedLastIndex)

        return theResult
    }

    
    // 注意：思いついても再帰を使わないこと。遅くなる上にメモリも食う。
    func incrementalIndexPaths(at length: Int) -> Array<IndexPath>
    {
        var theResult: Array<IndexPath> = []
        var theIndexPath: IndexPath = self
        
        for _ in 1...length
        {
            theResult.append(theIndexPath)
            theIndexPath = theIndexPath.indexPathByIncrementLastIndex()
        }
        
        return theResult
    }
    
    func indexPathTo(depth:IndexPath.Element) -> IndexPath?
    {
        let theLength = Swift.min(depth, self.count)

        if theLength < 1
        {
            return nil
        }

        return self.dropLast(self.count - theLength)
    }

    func isDescendant(_ indexPath: IndexPath) -> Bool
    {
        let theSubIndexPath = self.indexPathTo(depth:indexPath.count)
        
        if indexPath == theSubIndexPath
        {
            return true
        }
        
        return false
    }
    
    func isAncestor(_ indexPath: IndexPath) -> Bool
    {
        return indexPath.isDescendant(self)
    }
}
