//
//  TO2Node.swift
//  TreeOutlinerSwift
//
//  Created by narita on 2017/10/13.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

@objc(TO2Node)

class TO2Node: NSObject, NSCoding, NSCopying, NSPasteboardWriting, NSPasteboardReading
{
    // MARK: - class methods
    
    // MARK: - init methods
    override init()
    {
        
    }
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    func copy(with zone: NSZone? = nil) -> Any
    {
        let theNode = TO2Node()
        
        theNode.text = self.text
        
        return theNode
    }

    func compare(_ other: TO2Node) -> ComparisonResult
    {
        return self.text.compare(other.text)
    }

    // MARK: - NSCoding methods
    
    func encode(with aCoder: NSCoder)
    {
        aCoder.encode(self.text, forKey: "text")
        aCoder.encode(self.children, forKey: "children")
    }
    
    required init?(coder aDecoder: NSCoder)
    {
        self.text = aDecoder.decodeObject(forKey: "text") as! String
        self.children = aDecoder.decodeObject(forKey: "children") as! NSMutableArray
    }
    
    func writableTypes(for pasteboard: NSPasteboard) -> [NSPasteboard.PasteboardType]
    {
        return [TO2Node.TO2NodesPasteboardType]
    }
    
    func pasteboardPropertyList(forType type: NSPasteboard.PasteboardType) -> Any?
    {
        var theResult: Any?
        
        switch type
        {
            case TO2Node.TO2NodesPasteboardType:
                theResult = NSKeyedArchiver.archivedData(withRootObject: self)
            
            default:
                theResult = nil
        }
        
        return theResult
    }
    
    static func readableTypes(for pasteboard: NSPasteboard) -> [NSPasteboard.PasteboardType]
    {
        return [TO2Node.TO2NodesPasteboardType]
    }
    
    required init?(pasteboardPropertyList propertyList: Any, ofType type: NSPasteboard.PasteboardType)
    {
        switch type
        {
            case TO2Node.TO2NodesPasteboardType:
                let theRootObject = NSKeyedUnarchiver.unarchiveObject(with: propertyList as! Data) as! TO2Node
                self.text = theRootObject.text
                self.children = theRootObject.children

            default:
                print("unsupported pasteboardType \(type)")
        }
    }

    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - accessor methods (in pairs)
    
    @objc var isLeaf: Bool
    {
        get
        {
            return self.children.count == 0
        }
    }
    
    @objc var parent: TO2Node? = nil
    
    @objc dynamic var children: NSMutableArray = NSMutableArray()

    // childrenへのアクセッサ。parentへの値を設定するため

    @objc func countOfChildren() -> Int
    {
        return self.children.count
    }
    
    @objc func objectInChildrenAtIndex(_ i: Int) -> Any?
    {
        return self.children[i]
    }

    @objc public func insertObject(_ children: AnyObject, inChildrenAtIndex index:Int)
    {
        if let object: TO2Node = children as? TO2Node
        {
            object.parent = self
            self.children.insert(object, at: index)
        }
    }
    
    @objc public func removeObjectFromChildrenAtIndex(_ index:Int)
    {
        if let object: TO2Node = self.children[index] as? TO2Node
        {
            object.parent = nil
            
            self.children.removeObject(at: index)
        }
    }
    // ---------------------------
    
    @objc var text: String = "Untitled Text"
    
    static let TO2NodesPasteboardType: NSPasteboard.PasteboardType =
                                        NSPasteboard.PasteboardType(rawValue: "com.mindto01s.TO2NodesPasteboardType")
    // MARK: - Utility methods

    func descendant(at indexPath: IndexPath) -> TO2Node?
    {
        if indexPath.count == 0
        {
            return self
        }

        let  theIndex: IndexPath.Element = indexPath.first!
        
        if theIndex >= self.children.count
        {
            print("out of range TO2Node descendant(at:) \(self)")
            return nil
        }

        
        // 末尾再帰の終了条件
        if indexPath.count == 1
        {
            return self.children[theIndex] as? TO2Node
        }
        
        // 再帰
        return (self.children[theIndex] as! TO2Node).descendant(at: indexPath.dropFirst())
    }

}
