//
//  TO2ViewController+Outline.swift
//  TreeOutlinerSwift
//
//  Created by narita on 2017/10/19.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

extension TO2ViewController : NSOutlineViewDataSource, NSOutlineViewDelegate
{
    
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - NSOutlineViewDelegate methods
    
    // MARK: - NSOutlineViewDataSource methods
    
    // DnD用のデータ保存
    // memo: itemの型がNSTreeNodeでないのは、理由があるか知りたい
    @objc func outlineView(_ outlineView: NSOutlineView, pasteboardWriterForItem item: Any) -> NSPasteboardWriting?
    {
        if let theTreeNode = item as? NSObject
        {
            let theData = NSKeyedArchiver.archivedData(withRootObject:
                theTreeNode.value(forKeyPath: "representedObject")!)
            
            let thePBItem =  NSPasteboardItem()
            
            thePBItem.setData(theData, forType: TO2Node.TO2NodesPasteboardType)
            
            return thePBItem
        }
        
        return nil
    }
    
    // Dragセッション開始
    @objc func outlineView(_ outlineView: NSOutlineView,
                           draggingSession session: NSDraggingSession,
                           willBeginAt screenPoint: NSPoint,
                           forItems arg_draggedItems: [Any]) // 引数とメンバ変数が同じ名前になったのでarg_を追加した
    {
        self.draggingItems = arg_draggedItems as? [NSTreeNode]
    }
    
    // Dragセッション終了
    @objc func outlineView(_ outlineView: NSOutlineView,
                           draggingSession session: NSDraggingSession,
                           endedAt screenPoint: NSPoint,
                           operation: NSDragOperation)
    {
        if operation == .delete
        {
            self.treeController?.removeObjects(atArrangedObjectIndexPaths:
                self.draggingItems!.map(){ return $0.indexPath })
        }
        
        self.draggingItems = nil
    }
    
    // Drop可否の判定
    @objc func outlineView(_ outlineView: NSOutlineView,
                           validateDrop info: NSDraggingInfo,
                           proposedItem item: Any?,
                           proposedChildIndex index: Int) -> NSDragOperation
    {
        let theDraggingSource = info.draggingSource()
        let theDstDocument: NSDocument? = self.representedObject as? NSDocument
        var theSrcDocument: NSDocument? = nil
        
        // Drag元のドキュメントを算出する
        if let theDraggingSourceView = theDraggingSource as! NSOutlineView?
        {
            if let theViewController = theDraggingSourceView.dataSource as! TO2ViewController?
            {
                theSrcDocument = theViewController.representedObject as? NSDocument
            }
        }
        
        // 同じドキュメントならば移動
        if theDstDocument == theSrcDocument
        {
            // Drag中のNodeを自分自身のNode以下にDrop出来てはいけないので制約を設ける
            // つまり、Nodeは自分自身の中へは移動出来ない。
            let theDragIndexPaths = self.draggingItems!.map(){$0.indexPath}
            let theDropIndexPath  = self.dropPointIndexPath(item: item, index: index)
            
            for i in theDragIndexPaths
            {
                if theDropIndexPath.isDescendant(i)
                {
                    return []
                }
            }
            
            return .move
        }
        else
        {
            return .copy
        }
    }
    
    @objc func outlineView(_ outlineView: NSOutlineView,
                           acceptDrop info: NSDraggingInfo,
                           item: Any?,
                           childIndex index: Int) -> Bool
    {
        var theResult = false
        let theInsertPointIndexPath = self.dropPointIndexPath(item: item, index: index)
        let theDragOperation = self.outlineView(outlineView,
                                                validateDrop: info,
                                                proposedItem: item,
                                                proposedChildIndex: index)
        
        switch(theDragOperation)
        {
        case .move:
            self.treeController?.move( self.draggingItems!, to: theInsertPointIndexPath)
            theResult = true
            
        case .copy:
            // ペーストボードから読み込んだデータから配列を生成する
            let theArray: [TO2Node] = info.draggingPasteboard().pasteboardItems!.map
            {
                let theData = $0.data(forType: TO2Node.TO2NodesPasteboardType)
                
                return NSKeyedUnarchiver.unarchiveObject(with: theData!) as! TO2Node
            }
            
            let theIndexPaths: Array<IndexPath> = theInsertPointIndexPath.incrementalIndexPaths(at: theArray.count)
            self.treeController!.insert(theArray, atArrangedObjectIndexPaths: theIndexPaths)
            self.treeController!.setSelectionIndexPaths(theIndexPaths)
            
            theResult = true
            
        default:
            theResult = false
        }
        
        return theResult
    }
    
    // 挿入地点を求める
    func dropPointIndexPath(item: Any?, index: Int) -> IndexPath
    {
        var theInsertPointIndexPath: IndexPath = IndexPath(index: 0)
        
        if item == nil //  最上位のレベル
        {
            if index == -1
            {
                theInsertPointIndexPath = IndexPath(index:
                    (self.treeController?.arrangedObjects.children?.count)!)
            }
            else
            {
                theInsertPointIndexPath = IndexPath(index: index)
            }
        }
        else //  item以下のレベル
        {
            if let theTreeNode = item as? NSTreeNode
            {
                
                let theIndexPath: IndexPath = theTreeNode.value(forKeyPath: "indexPath") as! IndexPath
                
                if index == -1
                {
                    let theLastIndex = theTreeNode.children!.count
                    theInsertPointIndexPath = theIndexPath.appending(theLastIndex)
                }
                else
                {
                    theInsertPointIndexPath = theIndexPath.appending(index)
                }
            }
        }
        
        return theInsertPointIndexPath
    }
    
    // MARK: - accessor methods (in pairs)
    
    // MARK: - Utility methods
}

