//
//  ViewController.swift
//  TreeOutlinerSwift
//
//  Created by narita on 2017/10/13.
//  Copyright © 2017年 narita. All rights reserved.
//

import Cocoa

class TO2ViewController: NSViewController, MINTLJumpbarDelegate
{
    
    // MARK: - class methods
    
    // MARK: - init methods
    
    // MARK: - dealloc
    
    // MARK: - NSCopying, hash, isEqual:
    
    // MARK: - NSCoding methods
    
    // MARK: - restorableState methods
    
    // MARK: - life cycle methods
    
    override func viewDidLoad()
    {
        super.viewDidLoad()

        self.outlineView!.registerForDraggedTypes([TO2Node.TO2NodesPasteboardType])
        self.outlineView!.setDraggingSourceOperationMask([.copy, .delete], forLocal: false)
        self.outlineView!.setDraggingSourceOperationMask([.copy, .move], forLocal: true)
        
        self.jumpBar!.unregisterDraggedTypes()
        self.jumpBar!.registerForDraggedTypes([TO2Node.TO2NodesPasteboardType])
        self.jumpBar!.setDraggingSourceOperationMask([.link], forLocal: true)

        // DocumentのrootNodeが設定されたら反映するようにする
        self.documentRootObservation = observe(\.representedObject?)
        { _, _ in
            self.jumpBar!.setNeedsUpdateUI()
        }
        
        // 選択範囲が変わると呼び出すようにする
        self.selectedIndexObservation = observe(\.treeController?.selection)
        { _, _ in
            self.jumpBar!.setNeedsUpdateUI()
        }
    }
    
    // MARK: - action methods
    
    // MARK: - event handling methods
    
    // MARK: - drawing methods
    
    // MARK: - delegate/datasource methods
    
    // MARK: - MINTLJumpbarDelegate methods
    
    // 深さをdelegateに聞く。aa>bb>ccは深さ3。aaは深さ1。深さ0はありえない。
    @objc  func titleDepth(_ jumpbar: MINTLJumpbar) -> Int
    {
        if self.treeController?.selectionIndexPath != nil
        {
            if let theSelectedIndexPath = self.treeController!.selectionIndexPath! as IndexPath?
            {
                return theSelectedIndexPath.count
            }
        }
        
        return 0
    }
    
    // aa>bb>ccで、index=0の時は、"aa"を返す。index=2の時は"cc"を返す。0ベース。深さ以下。
    @objc  func jumpbar(_ jumpbar: MINTLJumpbar, titleIndex: Int) -> String
    {
        if let theDoc = self.representedObject as! TO2Document?
        {
            if self.treeController != nil
            {
                let theSelectedIndexPath = self.treeController!.selectionIndexPath!
                
                let thePartalIndexPath = theSelectedIndexPath.dropLast(
                                            (theSelectedIndexPath.count - titleIndex - 1) )
                
                let theNode: TO2Node = theDoc.rootNode!.descendant(at: thePartalIndexPath)!
                
                return theNode.text
            }
        }
        
        return "this is Error String"
    }

    @objc  func jumpbar(_ jumpbar: MINTLJumpbar, click index: Int)
    {
        // クリックした位置に選択範囲を変更する
        let theNowSelectionPath = self.treeController!.selectionIndexPath
        let theNewSelectionPath = theNowSelectionPath?.dropLast((theNowSelectionPath?.count)! - index - 1)
        
        self.treeController!.setSelectionIndexPath(theNewSelectionPath)
    }

    // 長クリックされた時に表示するメニューを返す
    // ToDo: subメニューもサポートしたい
    @objc func jumpbar(_ jumpbar: MINTLJumpbar, menuFrom index: Int) -> NSMenu?
    {
        let theNowSelectionPath = self.treeController!.selectionIndexPath
        let theNewSelectionPath = theNowSelectionPath?.dropLast((theNowSelectionPath?.count)! - index - 1)

        let theTreeNode = self.treeController?.arrangedObjects.descendant(at: theNewSelectionPath!)

        let theMenu = NSMenu()

        theTreeNode?.parent!.children?.forEach
        {
            let theMenuItem = NSMenuItem()
            let theTO2Node = $0.representedObject as! TO2Node?
            
            theMenuItem.title = theTO2Node!.text
            theMenuItem.target = self
            theMenuItem.action = #selector(jumpbarMenu(_ :))
            theMenuItem.isEnabled = true
            theMenuItem.state = theTreeNode == $0 ? .on : .off
            theMenuItem.representedObject = $0
                
            theMenu.addItem(theMenuItem)
        }
        
        return theMenu
    }

    @objc @IBAction func jumpbarMenu(_ sender: Any?)
    {
        if let theMenuItem = sender as! NSMenuItem?
        {
            if let theTreeNode = theMenuItem.representedObject as! NSTreeNode?
            {
                self.treeController?.setSelectionIndexPath(theTreeNode.indexPath)
            }
        }

    }
    
    // MARK: - accessor methods (in pairs)

    @IBOutlet var outlineView: NSOutlineView? = nil
    @IBOutlet var treeController: NSTreeController? = nil
    @IBOutlet var jumpBar: MINTLJumpbar? = nil
    
    override var representedObject: Any?
    {
        didSet
        {
            // Update the view, if already loaded.
        }
    }

    // Drag&DropのDrag元のindexPathsを保持するための物
    var draggingItems: Array<NSTreeNode>? = nil
    
   
    var selectedIndexObservation: NSKeyValueObservation?  // KVOでoutlineViewの選択範囲を監視する
    var documentRootObservation: NSKeyValueObservation?   // KVOでdocumentのrootNodeを監視する

    // MARK: - Utility methods

}

